; 赤外線データ定義フォーマット
;	[ヘッダ][コマンド][コマンド][コマンド]...
;
;		ヘッダ
;		  7   6   5   4   3   2   1   0
;		+---+---+---+---+---+---+---+---+
; +0	+ I |        TICKLEN            |
;		+---+---+---+---+---+---+---+---+
; +1	+  BIT0_2NDLEN  |  BIT0 1STLEN  |
;		+---+---+---+---+---+---+---+---+
; +3	+  BIT1_2NDLEN  |  BIT1 1STLEN  |
;		+---+---+---+---+---+---+---+---+
;
;		TICKLEN		1ティックのパルス数 10～127
;					NEC:21 AEHA:21 SONY:23
;		I			反転フラグ 0:ON先行 1:OFF先行
;					NEC:0  AEHA:0  SONY:1
;		BIT0_1STLEN	BIT値0 先行ティック数	1～15
;					NEC:1  AEHA:1  SONY:1
;		BIT0_2NDLEN	BIT値0 後行ティック数	1～15
;					NEC:1  AEHA:1  SONY:1
;		BIT1_1STLEN	BIT値1 先行ティック数	1～15
;					NEC:1  AEHA:1  SONY:1
;		BIT1_2NDLEN	BIT値1 後行ティック数	1～15
;					NEC:3  AEHA:3  SONY:2
;
;
;		コマンド ON/OFF		赤外線を ON または OFF にする
;		  7   6   5   4   3   2   1   0
;		+---+---+---+---+---+---+---+---+
; +0	+ 1 | C |       TICKS           |
;		+---+---+---+---+---+---+---+---+
;
;		C			0:OFF にする  1:ON にする
;		TICKS		ON または OFF にするティック数	1～63
;
;
;		コマンド END	送信の終了
;		  7   6   5   4   3   2   1   0
;		+---+---+---+---+---+---+---+---+
; +0	+               0               |
;		+---+---+---+---+---+---+---+---+
;
;
;		コマンド BYTE	バイト単位のデータ送信
;		  7   6   5   4   3   2   1   0
;		+---+---+---+---+---+---+---+---+
; +0	+ 0 | 0 |      DATA_LEN         |
;		+---+---+---+---+---+---+---+---+
; +1～	+             BYTE0             |
;		+---+---+---+---+---+---+---+---+
;		+             BYTE1             |
;		+---+---+---+---+---+---+---+---+
;		                :
;		DATA_LEN	データのバイト数	1～63(0 はコマンド END)
;		BYTEn		送信データ LSB first
;
;
;		コマンド BIT	ビット単位のデータ送信
;		  7   6   5   4   3   2   1   0
;		+---+---+---+---+---+---+---+---+
; +0	+ 0 | 1 | 0 | 0 | 0 |  BIT_LEN  |
;		+---+---+---+---+---+---+---+---+
; +1	+             DATA              |
;		+---+---+---+---+---+---+---+---+
;
;		BIT_LEN		データのビット数	1～7(0 は未定義)
;		DATA		送信データから指定ビット LSB first
;
;
;		コマンド BACK/THRU	リピート制御
;		  7   6   5   4   3   2   1   0
;		+---+---+---+---+---+---+---+---+
; +0	+ 0 | 1 | 1 | C |      WAIT     |
;		+---+---+---+---+---+---+---+---+
;
;		C			0:BACK
;						時間待ちをして、ボタンが離されてなければリピートポイントに戻って実行
;						(リピートポイントの初期値はコマンド群の先頭)
;					1:THRU
;						時間待ちをして、ボタンが離されてなければ次以降のコマンドを実行
;						リピートポイントは THRU の次に更新
;		WAIT		待ち時間 0～15、実際の待ち時間は (WAIT + 1) * 16T
;

#define _ON(tick)	(0xc0 | ((tick) & 0x3f))
#define _OFF(tick)	(0x80 | ((tick) & 0x3f))
#define _BYTE(len)	(0x00 | (len) & 0x3f)
#define	_BIT(len)	(0x40 | (len) & 0x07)
#define	_BACK(t)	(0x60 | (t) & 0x0f)
#define	_THRU(t)	(0x70 | (t) & 0x0f)
#define	_END		0x00

