; ==============================================================================
; 赤外線送信処理
; 使用レジスタ R16～R23 ほか
; ==============================================================================

; タイマー制御用レジスタ
.DEF INITIAL_LED = R19	
.DEF TICKS_BIT0 = R20	; BIT0 の先行/後行 ティック数(4+4 bit)
.DEF TICKS_BIT1 = R21	; BIT1 の先行/後行 ティック数(4+4 bit)

; 現ティック終了待ち
; ZF=0 であること
; マクロ直後の命令では既に ZF=0 になっている
.MACRO WAIT_TICK
WAIT_TICK_LP:
	brne WAIT_TICK_LP
.ENDM


	; ==============================================================================
	; IR送信処理
	; ARG: ボタン番号(0～)
	; ==============================================================================
SUB_SEND_IR:
	; データアドレス取得 X
	ldi YL, LOW(ADDR(DATA_PTR))
	ldi YH, HIGH(ADDR(DATA_PTR))
	add ARG, ARG
	add YL, ARG
	adc YH, ZERO
	ld XL, Y+
	ld XH, Y
	; パルス定義取得
	ld TICK_WIDTH, X+
	ldi INITIAL_LED, TCCR0A_LED_ON
	sbrc TICK_WIDTH, 7
	ldi INITIAL_LED, TCCR0A_LED_OFF	; 先行 OFF
	andi TICK_WIDTH, 0x7f
	ld TICKS_BIT0, X+
	ld TICKS_BIT1, X+

CMD_REPEAT_ENTRY:
	; 赤外線送信クロック
	ldi TMP, CLKPSR_CLK_SEND
	out CCP, CONST_CCPID
	out CLKPSR, TMP

	; リピート用にデータ先頭アドレス記録
	sts SRAM_REPEAT_ADDR_L, XL
	sts SRAM_REPEAT_ADDR_H, XH

	; パルスタイマクリア(初回はすぐタイムアウトするように、できるだけ少なく)
	ldi PULSE_TIMER, 5	; BYTEコマンド内最初の WAIT_TICK までの時間

	; タイマ ON
	ldi TMP, PRR_TIM_ON
	out PRR, TMP
	;out TCNT0H, ZERO	; 内部一時レジスタは 0 のはずなので使いまわす
	out TCNT0L, ZERO

	; タイマ割り込みフラグ
	ldi TMP, TIFR0_INT_PULSE
	out TIFR0, TMP
	out TIMSK0, TMP
	sei

	; コマンド処理ループ
	; Zフラグは当てにせずに処理をすること
CMD_LP:
	; ループ時パルスカウント
	; LEADER 後	2 pulse 6 cnt
	; BYTE 後	4 pulse 6 cnt	★最悪値
	; BIT 後	4 pulse 6 cnt

	; コマンド取り出し
	ld LP1, X+	; CMD

	sbrc LP1, 7
	rjmp CMD_LEADER

	;****************
	;* データ出力
	;****************
CMD_DATA:
	sbrc LP1, 6
	rjmp CMD_DATA_BIT

	;****************
	;* バイトデータ出力
	;****************
CMD_DATA_BYTE:
	; バイト数指定送信
	andi LP1, 0x3f
	subi LP1, 1
	brcs CMD_LP_BREAK

CMD_DATA_BYTE_LP1:
	; 8bit送信
	ldi LP2, 8-1 ; 8bit
	rcall SUB_SEND_BITS
	subi LP1, 1
	brcs CMD_LP
	rjmp CMD_DATA_BYTE_LP1

	;****************
	;* ビットデータ出力
	;****************
CMD_DATA_BIT:
	sbrc LP1, 5
	rjmp CMD_LP_BREAK
	; ビット数指定送信
	dec LP1
	andi LP1, 0x07
	mov LP2, LP1
	rcall SUB_SEND_BITS
	rjmp CMD_LP

	;****************
	;* リーダー送信
	;****************
CMD_LEADER:
	ldi ARG, TCCR0A_LED_OFF
	sbrc LP1, 6
	ldi ARG, TCCR0A_LED_ON
	andi LP1, 0x3f
	dec LP1

	; 指定長のティック出力
CMD_LEADER_LP:
	clz
	WAIT_TICK					; パルス位置 7 pulse 8 cnt
	subi LP1, 1
	brcs CMD_LP
	rjmp CMD_LEADER_LP

	;****************
	;* コマンド終了
	;* LP1 は 0xff(END時) または THRU/BACKコマンド
	;****************
CMD_LP_BREAK:
	; 次のティックで出力OFF
	ldi ARG, TCCR0A_LED_OFF
	;clz	ここに来るまでに必ず割り込み処理が入って ZF=0 になってる
	WAIT_TICK

	; 低速クロック 1/16
	ldi TMP, CLKPSR_CLK_REPEAT
	out CCP, CONST_CCPID
	out CLKPSR, TMP

	; 終了コマンドなら終了
	sbrc LP1, 7 ; 0xff(END時)判定, THRU/BACK なら bit7 は 0
	ret

	; リピート処理
	sbrc LP1, 4		; bit4 0:REPEAT 1:BACK
	rjmp CMD_REPEAT_WAIT
	; バックポイント復元
	lds XL, SRAM_REPEAT_ADDR_L
	lds XH, SRAM_REPEAT_ADDR_H

	;andi LP1, 0x0f	; CF ではなく HF を使うので不要
CMD_REPEAT_WAIT:
	; ボタンが離されるまで時間待ち
	sbic PINB, PNO_BTNIN
	ret ; ボタンが離された

	;clz	ここに来るまでに必ず割り込み処理が入って ZF=0 になってる
	WAIT_TICK ; クロックが 1/16 なので 16ティック分待つ

	subi LP1, 1
	brhs CMD_REPEAT_ENTRY
	rjmp CMD_REPEAT_WAIT



	; ==============================================================================
	; データを取得して送信
	; ※呼び出し元は周期 13 以内で rcall すること
	; ※PULSE_TIMER は 5以上残っていること
	; ※呼び出し元に戻った時点で次ティックを 2パルス消費済み、timer=5 になってる
	; X: 送信データアドレス
	; LP2: ビット数 - 1(0～7)
	; 破壊: TMP ARG LP2 R16-R17
	; ==============================================================================
SUB_SEND_BITS:
	.DEF CUR_DATA = R16
	.DEF CUR_TICKS = R17

	; 送信データ取得		
	ld CUR_DATA, X+
SEND_BITS_LP1:

	; LSB の値によってティック値選択
	mov CUR_TICKS, TICKS_BIT0
	sbrc CUR_DATA, 0
	mov CUR_TICKS, TICKS_BIT1
	dec CUR_TICKS	; ループ条件に ZF ではなく CF(HF) を使うため、とりあえず前半を 1 減らしておく
					; 0 にはならないから ZF=0 のはず

	; ティック前半
	mov ARG, INITIAL_LED
SEND_BITS_LP2:
	WAIT_TICK			; パルス位置 初回 BYTE 4 pulse 9 cnt / BYTE後 BYTE 9 pulse 7 cnt
	subi CUR_TICKS, 1
	brhc SEND_BITS_LP2	; 前4bit のカウント終了をハーフキャリーで判定、後4bit は 1減らされている

	; ティック後半
	ldi TMP, TCCR0A_LED_ON ^ TCCR0A_LED_OFF
	eor ARG, TMP ; ZF=0
SEND_BITS_LP3:
	WAIT_TICK
	subi CUR_TICKS, 0x10 ;ZF=0
	brcc SEND_BITS_LP3

	; 次のビット
	lsr CUR_DATA
	subi LP2, 1
	brcc SEND_BITS_LP1

	ret

	.UNDEF CUR_DATA
	.UNDEF CUR_TICKS

; タイマー制御用
.UNDEF INITIAL_LED
.UNDEF TICKS_BIT0
.UNDEF TICKS_BIT1
