;
; 20+1ボタン赤外線リモコン
; ATtiny10 500kHz
;

#define ADDR(l) (MAPPED_FLASH_START + 2 * l)
#define SRAM(a) (SRAM_START + (a))

; ピン定義
; PB0	LED(HZ/H /MISO)			LED出力(正論理、トランジスタ駆動)
; PB1	BTNOUT(OC0B/HZ /SCK)	ボタン読み取り電圧出力ピン（常にプルダウンされるラインなので、プルダウンに強い SCKピンに割り当てる）
; PB2	BTNIN(INT0/ADC2)		ボタン入力には INT0 が使える必要があるのでここ
; PB3	RESET
.EQU PNO_LED = 0
.EQU PNO_BTNOUT = 1
.EQU PNO_BTNIN = 2
.EQU PNO_RESET = 3

; 電力制御
.EQU PRR_OFF		= (1 << PRADC) | (1 << PRTIM0)	; ADC:OFF TIM0:OFF
.EQU PRR_ADC_ON		= (0 << PRADC) | (1 << PRTIM0)	; ADC:ON  TOM0:OFF
.EQU PRR_TIM_ON		= (1 << PRADC) | (0 << PRTIM0)	; ADC:OFF TOM0:ON

; システムクロック
.EQU CLKPSR_CLK_SEND	= 0x04		; 0100 赤外線送信時クロック 1/16 = 500kHz
.EQU CLKPSR_CLK_REPEAT	= 0x08		; 1000 低速クロック 1/256 = 31.25kHz
.EQU CLKPSR_CLK_SCANBTN	= 0x05		; 0101 ボタン待機時クロック 1/32 = 250kHz(ADCクロック確保のためここまでしか下げられない)

; ADC設定 CLKPSR_CLK_SCANBTN の 1/2 125kHz
.EQU ADCSRA_ADC_OFF	= (0 << ADEN) | (0 << ADSC) | (0 << ADATE) | (1 << ADIF) | (0 << ADIE) | (0 << ADPS2) | (0 << ADPS1) | (1 << ADPS0)
.EQU ADCSRA_ADC_ON	= (1 << ADEN) | (0 << ADSC) | (0 << ADATE) | (1 << ADIF) | (1 << ADIE) | (0 << ADPS2) | (0 << ADPS1) | (1 << ADPS0)

; WDT
.EQU WDTCSR_OFF		= 0	; ZERO でいい
.EQU WDTCSR_16ms	= (1 << WDIF) | (1 << WDIE) | (0 << WDE) | (0 << WDP3) | (0 << WDP2) | (0 << WDP1) | (0 << WDP0);
.EQU WDTCSR_32ms	= (1 << WDIF) | (1 << WDIE) | (0 << WDE) | (0 << WDP3) | (0 << WDP2) | (0 << WDP1) | (1 << WDP0);
.EQU WDTCSR_64ms	= (1 << WDIF) | (1 << WDIE) | (0 << WDE) | (0 << WDP3) | (0 << WDP2) | (1 << WDP1) | (0 << WDP0);

; SLEEPモード
.EQU SMCR_ADC		= (0 << SM2) | (0 << SM1) | (1 << SM0) | (1 << SE)
.EQU SMCR_POWERDOWN	= (0 << SM2) | (1 << SM1) | (0 << SM0) | (1 << SE)

; LEDキャリア周波数・デューティー比
; ON:OFF = 4:9  13clock周期 = 38.48kHz 30.8% duty
.EQU PULSE_L_CLK = 9
.EQU PULSE_H_CLK = 4

; LED出力定数
; WGA0(3-0): 1110 高速PWM TOP値 ICR0
; CS0(2-0): 001	  clk x1 (500kHz)
; COM0x(1-0): 00  LED = PORTB:PBx
; COM0x(1-0): 11  LED = OC0x(パルス L => H)
; x = PNO_LED によって 0:A 1:B
.EQU TCCR0B_VALUE	= (1 << WGM03) | (1 << WGM02) | (0 << CS02) | (0 << CS01) | (1 << CS00)
.EQU TCCR0A_LED_ON	= (1 << WGM01) | (0 << WGM00) | ((PNO_LED == 0) << COM0A1) | ((PNO_LED == 0) << COM0A0) | ((PNO_LED == 1) << COM0B1) | ((PNO_LED == 1) << COM0B0)
.EQU TCCR0A_LED_OFF	= (1 << WGM01) | (0 << WGM00) | (0 << COM0A1) | (0 << COM0A0) | (0 << COM0B1) | (0 << COM0B0)

; パルス割り込みビット PNO_LED によって 0:A 1:B
.EQU TIFR0_INT_PULSE =	((PNO_LED == 0) << OCF0A) | ((PNO_LED == 1) << OCF0B)

.IF PNO_LED == 0
	.EQU OCR0_LEDH = OCR0AH
	.EQU OCR0_LEDL = OCR0AL
.ELSE
	.EQU OCR0_LEDH = OCR0BH
	.EQU OCR0_LEDL = OCR0BL
.ENDIF

; レジスタ
; Z の上位は必ず 0 にする
.DEF TICK_WIDTH = R22	; 1T のパルス数
.DEF PULSE_TIMER = R23	; 現ティックの残りパルス数
.DEF ARG = R24
.DEF CONST_CCPID = R25
#define LP1 YL
#define LP2 YH
#define TMP ZL
#define ZERO ZH

; SRAM
.EQU SRAM_REPEAT_ADDR_L = SRAM(0)	; 赤外線コマンドリピートアドレス L
.EQU SRAM_REPEAT_ADDR_H = SRAM(1)	; 〃 H


.ORG 0
	rjmp	RESET	; 1	RESET
	reti									; 2 INT0 

	; ==============================================================================
	; パルス割り込み処理に ベクタ3～8 の未使用ベクタを流用
	; ティックが切り替わったら ZF=1, そうじゃなかったら ZF=0
	; タイマ割り込みが有効になっている間は ZF を使った通常の条件分岐はできない
	; ティックが切り替わった瞬間の ARG の値が出力されるので、それまでに ARG を確定して
	; ZF=1 になるまで待機すること
	; ティック継続時は割り込み処理で 9clk、ティック切り替わり時は 12 消費
	; ==============================================================================

	; ==============================================================================
	; 割り込みルーチン: 次のティックになった
	; ==============================================================================
NEXT_TICK:
	out TCCR0A, ARG							; 3 PCINT0
	mov PULSE_TIMER, TICK_WIDTH				; 4 TIM0_CAPT
	reti									; 5 TIM0_OVF

	; ==============================================================================
	; パルス割り込み
	; OVF の代わりに COMPx を利用、タイミング先取を兼ねる
	; ARG: 次のティックの出力ピン状態
	; ==============================================================================
.IF PNO_LED == 0
INT_PULSE:
	dec PULSE_TIMER							; 6 TIM0_COMPA
	breq NEXT_TICK							; 7 TIM0_COMPB
	reti									; 8 ANA_COMP
	reti									; 9 WDT
.ELSE
	reti									; 6 TIM0_COMPA
INT_PULSE:
	dec PULSE_TIMER							; 7 TIM0_COMPB
	breq NEXT_TICK							; 8 ANA_COMP
	reti									; 9 WDT
.ENDIF

	reti									; 10 LVM
	reti									; 11 ADC


	; ==============================================================================
	; 初期化
	; ==============================================================================
RESET:
	; RESET PIN, BTNIN プルアップ
	ldi TMP, (1 << PNO_RESET) | (1 << PNO_BTNIN)
	out PUEB, TMP

	; 定数初期化
	ldi ZERO, 0
	ldi CONST_CCPID, 0xd8

	; SP設定
	ldi TMP, LOW(RAMEND)
	out SPL, TMP

	; LED を OUT/L、BTNOUT を OUT/L に
	out PORTB, ZERO ; 全出力ポート L
	ldi TMP, (1 << PNO_LED) | (1 << PNO_BTNOUT)
	out DDRB, TMP

	; ADC
	ldi TMP, PNO_BTNIN
	out ADMUX, TMP	; ADC 入力ピン設定

	; アナログコンパレータ OFF
	sbi ACSR, ACD

	; タイマ初期化
	ldi TMP, TCCR0A_LED_OFF
	out TCCR0A, TMP
	ldi TMP, TCCR0B_VALUE
	out TCCR0B, TMP

	; OCR0x LED出力 L のクロック数 - 1(残りは H)
	out OCR0_LEDH , ZERO
	ldi TMP, PULSE_L_CLK - 1
	out OCR0_LEDL, TMP

	; タイマ TOP値(周期 - 1)
	; TOP値の更新によって OCRA0 と OCRB0 が確定するので、順序に注意
	;out ICR0H, ZERO	; 内部一時レジスタ使いまわしで省略
	ldi TMP, PULSE_L_CLK + PULSE_H_CLK - 1
	out ICR0L, TMP

	; タイマ割り込みフラグ
	; OCRA0, OCRB0, OVF0
	;out TIMSK0, ZERO ; (0 << ICIE0) | (0 << OCIE0B) | (0 << OCIE0A) | (0 << TOIE0)

	; ==============================================================================
	; メインループ
	;	パワーダウンしてボタンが押されるまで待つ
	; ==============================================================================
MAIN:
	; ボタン待機クロック
	ldi TMP, CLKPSR_CLK_SCANBTN
	out CCP, CONST_CCPID
	out CLKPSR, TMP

MAIN_LP1:
	; ADC/TIMER 電源OFF
	ldi TMP, PRR_OFF
	out PRR, TMP

	; SLEEPモード = パワーダウン
	ldi TMP, SMCR_POWERDOWN
	out CCP, CONST_CCPID
	out SMCR, TMP

	sei

	; ボタン解放待ち
	ldi TMP, WDTCSR_64ms
WAIT_BTN_RELEASE:
	sbic PINB, PNO_BTNIN
	rjmp WAIT_BTN
	; 時間待ち
	wdr
	out WDTCSR, TMP ; WDT割込み ON(CCP の操作不要)
	sleep
	out WDTCSR, ZERO ; WDT割込み OFF(CCP の操作不要)
	rjmp WAIT_BTN_RELEASE

WAIT_BTN:
	; INT0 割込み許可
	sbi EIFR, INT0
	sbi EIMSK, INT0
	; パワーダウン
	sleep
	cbi EIMSK, INT0	; INT0割込み禁止

	; ボタン読み取り
	rcall SUB_READ_BUTTON
	cpi ARG, (BTN_NUM + BTN_EXTRA)
	brcc MAIN_LP1 ; 不明なボタン

	; ボタン処理
	rcall SUB_SEND_IR
	out TIMSK0, ZERO

	rjmp MAIN


; ボタン処理と赤外線送信処理
.include "ir.asm"
.include "button.asm"

; リモコンデータ
.include "data_def.asm"
.include "data_inc.asm"
